<?php
namespace App\Http\Controllers;

header("Access-Control-Allow-Origin: *");

use Log;
use App\Model\RechargeApi;
use App\Helper\Helper;
use Illuminate\Http\Request;
use App\Jobs\UpdateChargeDiscountJob;
use Config;



class ChargeDiscountApplyController extends Controller {

    private $rechargeObj;
    private $logDirectory;
    private $logFiles;
    private $logPath;
    private $helperObj;
    private $api;

    public function __construct() {      
        
        $this->rechargeWebhookSecret = env('RECHARGE_API_CLIENT_SECRET',true);
        $this->rechargeObj   = new RechargeApi();
        $this->helperObj  = new Helper();
        
        $this->setPaths();
        $this->getLogFilesList();
        $this->setWithLogFiles();

        date_default_timezone_set('US/Eastern');
    }

    public function setPaths() {

        $this->logDirectory = dirname(__FILE__) . "/../../../storage/logs/";
    }

    public function getLogFilesList() {

        $this->logPath = $this->logDirectory . date('dmY');

        if (!is_dir($this->logPath)) {
            mkdir($this->logPath);
            chmod($this->logPath, 0777);

        }

        $this->logFiles = array(
            'checkoutLineitemUpdate'   => array(
                "type"     => "text",
                "filename" => $this->logPath . "/log-checkoutLineitemUpdate.txt",
                "mode"     => ""
            ),
            'log'   => array(
                "type"     => "text",
                "filename" => $this->logPath . "/log-chargeDiscountUpdate.txt",
                "mode"     => ""
            )               
        );
    }

    public function setWithLogFiles() {

        if (!empty($this->logFiles)) {
            foreach ($this->logFiles as $fileDetails) {
                $fileName = $fileDetails['filename'];
                if ($fileDetails['type'] == 'csv' && !file_exists($fileName)) {
                    $handler = fopen($fileName, $fileDetails['mode']);
                    fputcsv($handler, $fileDetails['columns']);
                    fclose($handler);
                    touch($fileName);
                    chmod($fileName, 0777);
                }
            }

        }
    }

    public function updateNextChargeDiscount() {
        $this->helperObj->logMe("processed", $this->logFiles['log']['filename']);

        $received_digest = $_SERVER['HTTP_X_RECHARGE_HMAC_SHA256'];
        $requestData = file_get_contents('php://input');
        $verified = $this->verify_recharge_webhook($requestData, $received_digest);
        $verified = true;

        if ($verified) {
            $receivedData = json_decode($requestData);
            $this->helperObj->logMe("Order:- ".json_encode($receivedData), $this->logFiles['log']['filename']);
        
            if(!isset($receivedData->order->address_id) || empty($receivedData->order->address_id)) {
               
              $this->helperObj->logMe("Error in the webhook response -
                Missing required parameters in the webhook response or the
                response is not correct", $this->logFiles['log']['filename']);

            } else if (!isset($receivedData->order->discount_codes) || empty($receivedData->order->discount_codes)) {
                $this->helperObj->logMe($receivedData->order->id ." :- No discount code is applied in the order", $this->logFiles['log']['filename']);
            
            } else {
                $queueResponse = dispatch(new UpdateChargeDiscountJob($receivedData->order, $this->logFiles['log']['filename']));
            }
        } else {

            $this->helperObj->logMe('Recharge order create Webhook validation failed',
            $this->logFiles['log']['filename']);
        }

    }
    
    public function verify_recharge_webhook($data, $received_digest) {

        $calculated_digest = hash('sha256',$this->rechargeWebhookSecret.$data);
        if (hash_equals($received_digest,$calculated_digest)) {
            return true;
        } else {
            return false;
        }
    }   
}
