<?php
namespace App\Http\Controllers;

header("Access-Control-Allow-Origin: *");

use Log;
use App\Model\RechargeApi;
use App\Helper\Helper;
use Illuminate\Http\Request;
use Config;



class CouponValidityController extends Controller {

    private $rechargeObj;
    private $logDirectory;
    private $logFiles;
    private $logPath;
    private $helperObj;
    private $api;

    public function __construct() {      
        
        $this->rechargeObj   = new RechargeApi();
        $this->helperObj  = new Helper();
        
        $this->setPaths();
        $this->getLogFilesList();
        $this->setWithLogFiles();

        date_default_timezone_set('US/Eastern');
    }

    public function setPaths() {

        $this->logDirectory = dirname(__FILE__) . "/../../../storage/logs/";
    }

    public function getLogFilesList() {

        $this->logPath = $this->logDirectory . date('dmY');

        if (!is_dir($this->logPath)) {
            mkdir($this->logPath);
            chmod($this->logPath, 0777);

        }

        $this->logFiles = array(
            'couponValidityCheck'   => array(
                "type"     => "text",
                "filename" => $this->logPath . "/log-couponValidity.txt",
                "mode"     => ""
            )            
        );
    }

    public function setWithLogFiles() {

        if (!empty($this->logFiles)) {
            foreach ($this->logFiles as $fileDetails) {
                $fileName = $fileDetails['filename'];
                if ($fileDetails['type'] == 'csv' && !file_exists($fileName)) {
                    $handler = fopen($fileName, $fileDetails['mode']);
                    fputcsv($handler, $fileDetails['columns']);
                    fclose($handler);
                    touch($fileName);
                    chmod($fileName, 0777);
                }
            }

        }
    }
   
    public function checkCodeValidity(request $data) {
        
       
        if (empty($_REQUEST['data'])) {

            $message = "Endpoint parameters are missing";
            $error = "Bad request";
            $status = "Failed";
            $response = array(
                "status"       => $status,
                "message" => $message,
                "error" => $error
            );
            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die();
        }

        $requestData = $_REQUEST['data'];

        $receivedData = json_decode($requestData);

        if (!isset($receivedData->discount_code) || empty($receivedData->discount_code)) {

            $message = "Missing required parameter discount_code";
            $error = "Bad request";
            $status = "Failed";
            $response = array(
                "status"       => $status,
                "message" => $message,
                "error" => $error
            );
            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die();
        }

        $discountCode = trim($receivedData->discount_code);

        $discountDetails = $this->rechargeObj->getDiscountDetailsByDiscountCode($discountCode);
       
        if (!isset($discountDetails) || !isset($discountDetails->discounts) || isset($discountDetails->error)
         || isset($discountDetails->errors) || empty($discountDetails->discounts) || empty($discountDetails->discounts[0])) {

            $status = "Failed";
            $error  = "Unknown error";
            if (isset($discountDetails->error)) {

                $error = $discountDetails->error;

            } else if (isset($discountDetails->errors)) {

                $error = $discountDetails->errors;
            }
            $message = "Failed to get the discount details from recharge";

            $this->helperObj->logMe(
                "Failed to get the discount details from recharge: ".
                " | discountCode -> ".$discountCode.
                " | Error -> ".$error,
                $this->logFiles['couponValidityCheck']['filename']
            );

            $response = array(
                "status"       => $status,
                "message" => $message,
                "error" => $error
            );
            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die();
        }
        
        $endDate = $discountDetails->discounts[0]->ends_at;
        if ($endDate == null) {
            $status = "Valid";
            $message = "The coupon is valid.";
            $error  = "";

            $this->helperObj->logMe(
                "The coupon is valid. ".
                " | discountCode -> ".$discountCode.
                " | Error -> ".$error,
                $this->logFiles['couponValidityCheck']['filename']
            );

        } else {
            $currentDate = date('Y-m-d h:i:s');

            if (strtotime($currentDate) > strtotime($endDate)) {
                $status = "Invalid";
                $message = "The coupon is expired.";
                $error  = "";

                $this->helperObj->logMe(
                    "The coupon is expired. ".
                    " | discountCode -> ".$discountCode.
                    " | Error -> ".$error,
                    $this->logFiles['couponValidityCheck']['filename']
                );

            } else {
                $status = "Valid";
                $message = "The coupon is valid.";
                $error  = "";

                $this->helperObj->logMe(
                    "The coupon is valid. ".
                    " | discountCode -> ".$discountCode.
                    " | Error -> ".$error,
                    $this->logFiles['couponValidityCheck']['filename']
                );
            }
        }

        $response = array(
            "status"       => $status,
            "message" => $message,
            "error" => $error
        );
        echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
        die();
    }

    
}
