<?php

namespace App\Http\Controllers;

use App\Helper\Helper;
use Illuminate\Http\Request;
use App\Model\IterableApi;
use App\Helper\ResponseHelper;
use Log;
use App\Jobs\IterableSubscriptionCreateEvent;
use App\Jobs\IterableSubscriptionCancelEvent;
use App\Jobs\IterableSubscriptionChargeUpcomingEvent;
use App\Jobs\IterableOnetimeCreateEvent;

class IterableWebhookController extends Controller
{
    public $iterableObj;
    public $helper;
    public $rechargeWebhookSecret;
    
    private $logDirectory;
    private $logFiles;
    private $logPath;
    private $helperObj;

    public function __construct()
    {
        $this->iterableObj = new IterableApi();
        $this->helper = new ResponseHelper();
        $this->helperObj = new Helper();
        $this->rechargeWebhookSecret = env('RECHARGE_API_CLIENT_SECRET', true);
        
        $this->setPaths();
        $this->getLogFilesList();
        $this->setWithLogFiles();
    }
    
    public function setPaths() {

        $this->logDirectory = dirname(__FILE__) . "/../../../storage/logs/";
    }

    public function getLogFilesList() {

        $this->logPath = $this->logDirectory . date('dmY');
        if (!is_dir($this->logPath)){

         //Directory does not exist, so lets create it.
         mkdir($this->logPath);
         chmod($this->logPath, 0777);

        }
        $this->logFiles = array(
            'log' => array(
                "type" => "text",
                "filename" => $this->logPath . "/webhook-log.txt",
                "mode" => ""),
            );
    }
    
    public function setWithLogFiles() {

        if(!empty($this->logFiles)) {
            foreach ($this->logFiles as $fileDetails) {
                $fileName = $fileDetails['filename'];
                if($fileDetails['type'] == 'csv' && !file_exists($fileName)) {
                   $handler = fopen($fileName, $fileDetails['mode']);
                   fputcsv($handler, $fileDetails['columns']);
                   fclose($handler);
                   touch($fileName);
                   chmod($fileName, 0777);
                }
            }

        }
   }

    public function subscriptionCreatedEventWebhook()
    {
        $this->helperObj->logMe("Reached to subscription creation webhook", $this->logFiles['log']['filename']);
        $received_digest = $_SERVER['HTTP_X_RECHARGE_HMAC_SHA256'];
        $requestData = file_get_contents('php://input');
        $verified = $this->verifyRechargeWebhook($requestData, $received_digest);

        if ($verified) {
            $receivedData = json_decode($requestData);
            if (empty($receivedData->subscription->id)) {
                $this->helperObj->logMe("Error in the webhook response(Created subscription) -
                Missing required parameters in the webhook response or the
                response is not correct", $this->logFiles['log']['filename']);
                
            } else {
                $queueResponse = dispatch(new IterableSubscriptionCreateEvent($receivedData, $this->logFiles['log']['filename']));
            }
        } else {
            $this->helperObj->logMe("Recharge subscription create Webhook validation failed - ".$requestData, $this->logFiles['log']['filename']);
        }
    }

    public function subscriptionCanceledEventWebhook()
    {
        $this->helperObj->logMe("Reached to subscription Cancel webhook", $this->logFiles['log']['filename']);
        $received_digest = $_SERVER['HTTP_X_RECHARGE_HMAC_SHA256'];
        $requestData = file_get_contents('php://input');
        $verified = $this->verifyRechargeWebhook($requestData, $received_digest);

        if ($verified) {
            $receivedData = json_decode($requestData);
            if (empty($receivedData->subscription->id)) {
                $this->helperObj->logMe("Error in the webhook response(Cancel subscription) -
                Missing required parameters in the webhook response or the
                response is not correct", $this->logFiles['log']['filename']);
            } else {
                $queueResponse = dispatch(
                    new IterableSubscriptionCancelEvent($receivedData->subscription->customer_id, $this->logFiles['log']['filename'])
                );
            }
        } else {
            $this->helperObj->logMe("Recharge subscription cancelled Webhook validation failed - ".$requestData, $this->logFiles['log']['filename']);
        }
    }

     /**
     * Create Iterable event for upcoming charges
     */

    public function upcomingChargesEventWebhook()
    {
        $this->helperObj->logMe("Reached to upcoming charge webhook", $this->logFiles['log']['filename']);
        $received_digest = $_SERVER['HTTP_X_RECHARGE_HMAC_SHA256'];
        $requestData = file_get_contents('php://input');
        $verified = $this->verifyRechargeWebhook($requestData, $received_digest);

        if ($verified) {
            $receivedData = json_decode($requestData, true);

            if (empty($receivedData['charge']['id'])) {
                $this->helperObj->logMe("Error in the webhook response(upcoming charge) -
                Missing required parameters in the webhook response or the response is not correct", $this->logFiles['log']['filename']);
                
            } else {
                $queueResponse = dispatch(new IterableSubscriptionChargeUpcomingEvent($receivedData, $this->logFiles['log']['filename']));
            }
        } else {
           $this->helperObj->logMe("Recharge upcoming charge Webhook validation failed - ".$requestData,$this->logFiles['log']['filename']);
        }
    }
    
    
    public function orderCreateEventWebhook() {
        $this->helperObj->logMe("Reached to order creation webhook", $this->logFiles['log']['filename']);
        $received_digest = $_SERVER['HTTP_X_RECHARGE_HMAC_SHA256'];
        $requestData = file_get_contents('php://input');
        $verified = $this->verifyRechargeWebhook($requestData, $received_digest);
        if ($verified) {
            
            $this->helperObj->logMe($requestData, $this->logFiles['log']['filename']);

            $receivedData = json_decode($requestData);
            
                
            if(empty($receivedData->order->id)) {

                $this->helperObj->logMe("Error in the webhook response(Created OTP) -
                Missing required parameters in the webhook response or the
                response is not correct", $this->logFiles['log']['filename']);
            } else {
                
                if($receivedData->order->tags){
                    $tags = explode(',', $receivedData->order->tags);
                    if(!empty($tags) && in_array('OTP', $tags)){
                        
                        $this->helperObj->logMe("OTP order received => ".$receivedData->order->id, $this->logFiles['log']['filename']);
                        
                    	$queueResponse = dispatch(new IterableOnetimeCreateEvent($receivedData, $this->logFiles['log']['filename']));
                		ignore_user_abort(true);
                		header("Content-Length: 0");
                		header("Connection: Close");
                		flush();
                    }else{
                        $this->helperObj->logMe("Subscription order received => ".$receivedData->order->id, $this->logFiles['log']['filename']);
                    }
                    
                }

                
            }
        } else {

            $this->helperObj->logMe('Recharge order creation Webhook validation failed'.$requestData,
            $this->logFiles['log']['filename']);
        }
    }

    public function verifyRechargeWebhook($data, $received_digest)
    {

        $calculated_digest = hash('sha256', $this->rechargeWebhookSecret.$data);
        if (hash_equals($received_digest, $calculated_digest)) {
            return true;
        } else {
            return false;
        }
    }
}
