<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Model\IterableApi;
use App\Helper\ResponseHelper;
use Log;

class SurveyController extends Controller
{
    public $iterableObj;
    public $helper;

    public function __construct()
    {
        $this->iterableObj = new IterableApi();
        $this->helper = new ResponseHelper();
    }

    public function updateCustomer(Request $request)
    {
        $email = $request->input('email');
        $properties = $request->input('properties');

        if (empty($email)) {
            $returnData = [
                'error' => 'Empty Email'
            ];
            return $this->helper->returnData(600, "error", $returnData);
        }

        if (empty($properties)) {
            $returnData = [
                'error' => 'Empty Custom properties'
            ];
            return $this->helper->returnData(600, "error", $returnData);
        }

        $customProperties = [];
        foreach ($properties as $key => $value) {
            $customProperties[$key] = $value;
        }
        $userIterableUpdateRequest = array(
            "email" => $email,
            "dataFields" => array(
                "customProperties" => $customProperties
            )
        );
        // print_r($userIterableUpdateRequest); die;
        $userIterableUpdateResponse = $this->iterableObj->updateUserDataFields(json_encode($userIterableUpdateRequest));

        if (strtolower(trim($userIterableUpdateResponse->code)) == 'success') {
            Log::info("Successfully updated the customer properties - ".$email);
            
            $returnData = [
                'success' => "Successfully updated the customer properties - ".$email
            ];

            return $this->helper->returnData(200, "success", $returnData);
        } else {
            Log::info("Failed to update the customer properties - ".$email);
            Log::info(json_encode($userIterableUpdateResponse));

            $returnData = [
                'error' => "Failed to update the customer properties - ".$email
            ];

            return $this->helper->returnData(600, "error", $returnData);
        }
    }

    public function quizCompletionEvent(Request $request) {
        $email = $request->input('email');

        if (empty($email)) {
            $returnData = [
                'error' => 'Empty Email'
            ];
            return $this->helper->returnData(600, "error", $returnData);
        }

        $eventRequest = array(
            "events" => array(
                array(
                    "email" => strtolower($customerDetails->customer->email),
                    "eventName" => "Quiz Completion"
                )
            )
        );

        $eventResponse = $this->iterableObj->createNewIterableEvent(json_encode($eventRequest));

        if (isset($eventResponse->successCount) || $eventResponse->successCount > 0) {
            Log::info("Cancelled subscription iterable event - Successfully created event for Email (".
            strtolower($customerDetails->customer->email).")");
        } else {
            Log::info("Cancelled subscription iterable event - Failed to create event for Email (".
            strtolower($customerDetails->customer->email).")");
        }

        
    }
}
