<?php

namespace App\Jobs;

use App\Model\IterableApi;
use App\Model\RechargeApi;
use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Queue\InteractsWithQueue;
use Log;
use App\Helper\Helper;

class IterableSubscriptionCreateEvent implements ShouldQueue
{
    use InteractsWithQueue, Queueable;

    private $iterableObj;
    private $rechargeObj;
    private $customerID;
    private $properties;
    private $requestData;
    private $helperObj;
    private $logFile;
    

    /**
     * Create a new job instance.
     *
     * @return void
     */
    public function __construct($requestData, $logPath)
    {
        $this->requestData = $requestData;
        $this->customerID = $this->requestData->subscription->customer_id;
        $this->properties = $this->requestData->subscription->properties;
        $this->iterableObj = new IterableApi();
        $this->rechargeObj = new RechargeApi();
        $this->helperObj = new Helper();
        $this->logFile = $logPath;
    }

    public function handle()
    {

        $customerDetails = $this->rechargeObj->getRechargeCustomerDetailsByID($this->customerID);
        $this->helperObj->logMe("Reached to subscription create job", $this->logFile);
        $this->helperObj->logMe("customerDetails".json_encode($customerDetails), $this->logFile);
        
        if (isset($customerDetails->error) || isset($customerDetails->errors) ||
            empty($customerDetails->customer->email) || empty($customerDetails)) {
            $status = "failed";
            $error = "Unknown error";

            if (isset($customerDetails->error)) {
                $error = $customerDetails->error;
            } elseif (isset($customerDetails->errors)) {
                $error = "Request is not correct or missing the required parameters";
            }
            $this->helperObj->logMe("Created subscription iterable event - Failed to get the Recharge user details for CustomerID (" .
                $this->customerID . ")", $this->logFile);

            die();
        }

        $eventRequest = array(
            "events" => array(
                array(
                    "email" => strtolower($customerDetails->customer->email),
                    "eventName" => "subscriptionCreated",
                ),
            ),
        );

        $userIterableUpdateRequest = array(
            "email" => $customerDetails->customer->email,
            "dataFields" => array(
                "rechargeHash" => $customerDetails->customer->hash,
            ),
        );

        $userIterableUpdateResponse = $this->iterableObj->updateUserDataFields(json_encode($userIterableUpdateRequest));

        if (strtolower(trim($userIterableUpdateResponse->code)) == 'success') {
            
            $this->helperObj->logMe("User rechargeHash and subscription properties - Successfully
            updated the user recharge Hash and properties in Iterable for Email (" .
                strtolower($customerDetails->customer->email) . ")", $this->logFile);
            
        } else {
             $this->helperObj->logMe("User rechargeHash and subscription properties - Failed to update
            the user recharge Hash and properties in Iterable for Email (" .
                strtolower($customerDetails->customer->email) . ")", $this->logFile);
                
            Log::info();
        }
        
        $this->helperObj->logMe("eventRequest".json_encode($eventRequest), $this->logFile);
      
        $eventResponse = $this->iterableObj->createNewIterableEvent(json_encode($eventRequest));
        
        if (isset($eventResponse->successCount) && $eventResponse->successCount > 0) {
            
            $this->helperObj->logMe("Created subscription iterable event - Successfully created event for Email (" .
                strtolower($customerDetails->customer->email) . ")".json_encode($eventResponse), $this->logFile);
            
        } else {
            $this->helperObj->logMe("Created subscription iterable event - Failed to create event for Email (" .
                strtolower($customerDetails->customer->email) . ")".json_encode($eventResponse), $this->logFile);

        }
    }
}
