<?php

namespace App\Jobs;

use Illuminate\Bus\Queueable;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Contracts\Queue\ShouldQueue;
use App\Model\RechargeApi;
use App\Model\IterableApi;
use Log;

class IterableSubscriptionCreateEvent implements ShouldQueue
{
    use InteractsWithQueue, Queueable;

    private $iterableObj;
    private $rechargeObj;
    private $customerID;
    private $properties;
    private $requestData;

    /**
     * Create a new job instance.
     *
     * @return void
     */
    public function __construct($requestData)
    {
        $this->requestData = $requestData;
        $this->customerID  = $this->requestData->subscription->customer_id;
        $this->properties = $this->requestData->subscription->properties;
        $this->iterableObj = new IterableApi();
        $this->rechargeObj = new RechargeApi();
    }

    public function handle()
    {

        $customerDetails = $this->rechargeObj->getRechargeCustomerDetailsByID($this->customerID);
        Log::info("customerDetails");
        Log::info(json_encode($customerDetails));
        if (isset($customerDetails->error) || isset($customerDetails->errors) ||
            empty($customerDetails->customer->email) || empty($customerDetails)) {
            $status = "failed";
            $error = "Unknown error";

            if (isset($customerDetails->error)) {
                $error = $customerDetails->error;
            } elseif (isset($customerDetails->errors)) {
                $error = "Request is not correct or missing the required parameters";
            }

            Log::info("Created subscription iterable event - Failed to get the Recharge user details for CustomerID (".
            $this->customerID.")");

            die();
        }

        $eventRequest = array(
            "events" => array(
                array(
                    "email" => strtolower($customerDetails->customer->email),
                    "eventName" => "Subscription created"
                )
            )
        );
        Log::info("eventRequest");
        Log::info(json_encode($eventRequest));
        $eventResponse = $this->iterableObj->createNewIterableEvent(json_encode($eventRequest));
        Log::info("eventResponse");
        Log::info(json_encode($eventResponse));
        if (isset($eventResponse->successCount) || $eventResponse->successCount > 0) {
            Log::info("Created subscription iterable event - Successfully created event for Email (".
            strtolower($customerDetails->customer->email).")");
        } else {
            Log::info("Created subscription iterable event - Failed to create event for Email (".
            strtolower($customerDetails->customer->email).")");

        }
    }
}
