<?php
namespace App\Jobs;

use Illuminate\Bus\Queueable;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Contracts\Queue\ShouldQueue;
use App\Model\RechargeApi;
use App\Model\IterableApi;
use App\Helper\Helper;

use Osiset\BasicShopifyAPI\BasicShopifyAPI;
use Osiset\BasicShopifyAPI\Options;
use Osiset\BasicShopifyAPI\Session;

class ShopifyCustomer implements ShouldQueue
{
    use InteractsWithQueue, Queueable;

    private $helperObj;
    private $requestData;
    private $logFile;

    private $api;

    /**
     * Create a new job instance.
     *
     * @return void
     */
    public function __construct($requestData, $logPath)
    {
        $this->requestData = $requestData;
        $this->logFile = $logPath;
        
        $options = new Options();
        $options->setType(true); // Makes it private
        $options->setVersion(env('SHOPIFY_API_VER'));
        $options->setApiKey(env('SHOPIFY_API_KEY'));
        $options->setApiPassword(env('SHOPIFY_API_PASSWORD'));
        
        $this->api = new BasicShopifyAPI($options);
        // $this->api->setSession(new Session($_SESSION['shop']));
        
        $this->helperObj = new Helper();
    }

    public function handle()
    {
        $this->helperObj->logMe("Reached to customer create job", $this->logFile);
        
        if(isset($this->requestData->email)){
            
            $customerAvailable = false;
            
            $result = $this->api->rest('GET',
                    htmlentities('/admin/api/'.env('SHOPIFY_API_VER').'/customers/search.json?query=email:'.$this->requestData->email), null);

            if (!isset($result->body->customers[0]) || empty($result->body->customers[0])) {
                
                $this->helperObj->logMe("No such customer found in shopify (search) -".$this->requestData->email, $this->logFile);
                
            }else{
                foreach ($result->body->customers as $cust) {
                    if ($cust->email == $this->requestData->email) {
                       $customerAvailable = true;
                    }
                }
            }

            
            if($customerAvailable){
                $params = array(
                    'customer' => array(
                        'email' => $this->requestData->email,
                        'accepts_marketing' => 'true'
                    )
                );
                $retry = false;
                $try = 1;
                do{
                    $this->helperObj->logMe($try.") Trying to create customer in shopify -".$this->requestData->email, $this->logFile);
                    
                    $result = $this->api->rest('POST', '/admin/api/'.$this->shopify_api_version.'/customers.json', $params);
                    if(!isset($result->body->customer->id) || !isset($result->errors->body->errors)){
                        $retry = true;
                        $try++;
                    }
        
                } while($retry && $try <=3);
                
                if (isset($result->body->customer->id)) {
                    
                    $this->helperObj->logMe("Successfully created customer -".$this->requestData->email, $this->logFile);
                    
                } else {
                    
                    if (isset($result->errors->body->errors)) {
                        
                       $this->helperObj->logMe("Failed to created customer -".$this->requestData->email." ".$result->errors->body->errors, $this->logFile);
    
                    } else {
                        
                        $this->helperObj->logMe("Succesfully created customer".$this->requestData->email, $this->logFile);
                        
                    }
    
                }
            }           

        }
        
    }

}