<?php
namespace App\Model;

use Illuminate\Database\Eloquent\Model;
use Log;

class RechargeApi extends Model
{

    public $rechargeUrl;
    public $accessToken;
    public $rechargeObj;
    public $rechargeApiKey;

    public function __construct()
    {

        $this->rechargeUrl = env('RECHARGE_URL', true);
        $this->rechargeApiKey = env('RECHARGE_API_KEY', true);
    }

    public function getChargeById($id)
    {

        $method = 'GET';
        $path = $this->rechargeUrl . 'charges/'.$id;
        $response = $this->call($method, $path);
        return json_decode($response);
    }

    /**
     * update subscription by ID
     */
    public function updateSubscription($subscription_id, $data = null)
    {

        $method = 'PUT';
        $path = env('RECHARGE_URL', true).'/subscriptions/'.$subscription_id;
        $response = $this->call($method, $path, $data);
        return json_decode($response);
    }

    /**
     * get subscription by subscription_id
     */
    public function getSubscription($subscription_id)
    {

        $method = 'GET';
        $path = $this->rechargeUrl . '/subscriptions/' . $subscription_id;
        $response = $this->call($method, $path);
        return json_decode($response);
    }

    /**
     * get subscription by subscription_id
     */
    public function activateSubscription($subscription_id)
    {

        $method = 'POST';
        $path = $this->rechargeUrl . '/subscriptions/' . $subscription_id . '/activate';
        $data="{}";
        $response = $this->call($method, $path, $data);
        return json_decode($response);
    }

    /**
     * create recharge checkout
     */
    public function createCheckout($data)
    {

        $method = 'POST';
        $path = env('RECHARGE_URL', true).'/checkouts';
        $curl = curl_init();
        curl_setopt($curl, CURLOPT_URL, $path);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_ENCODING, '');
        curl_setopt($curl, CURLOPT_MAXREDIRS, 10);
        curl_setopt($curl, CURLOPT_TIMEOUT, 30);
        curl_setopt($curl, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_1);
        curl_setopt($curl, CURLOPT_CUSTOMREQUEST, $method);

        if ($method !== 'GET') {
            curl_setopt($curl, CURLOPT_POSTFIELDS, $data);
        }

        curl_setopt($curl, CURLOPT_HTTPHEADER, array(
            "cache-control: no-cache",
            "x-recharge-access-token: " . env('RECHARGE_API_KEY', true),
            "content-type: application/json",
        ));

        $response = curl_exec($curl);
        //$response = $this->call($method, $path, $data);
        return json_decode($response);
    }

    /**
     * update recharge checkout
     */
    public function updateCheckout($token, $data)
    {

        $method = 'PUT';
        $path = env('RECHARGE_URL', true).'/checkouts/'.$token;
        $response = $this->call($method, $path, $data);
        return json_decode($response);
    }

    /**
     * get recharge checkout
     */
    public function retrieveCheckout($token, $status = 'checkout')
    {

        $method = 'GET';
        if ($status == 'checkout') {
            $path = env('RECHARGE_URL', true).'/checkouts/'.
            $token;
        } else {
            $path = env('RECHARGE_URL', true).'/checkouts/'.
                $token.'/shipping_rates';
        }
        $response = $this->call($method, $path);
        return json_decode($response);
    }

    /**
     * update shipping line item
     */
    public function updateShippingLine($token, $data)
    {

        $method = 'PUT';
        $path = env('RECHARGE_URL', true).'/checkouts/'.$token;
        $response = $this->call($method, $path, $data);
        return json_decode($response);
    }

    /**
     * get customer details by customer id
     */
    public function getRechargeCustomerDetailsByID($customerID)
    {

        $method = 'GET';
        $path = $this->rechargeUrl . '/customers/'.$customerID;
        Log::info('path = '.$path);
        $response = $this->call($method, $path);
        Log::info('response = ');
        Log::info($response);

        return json_decode($response);
    }

    /**
     * get customer details by customer email
     */
    public function getRechargeCustomerDetailsByEmail($email)
    {

        $method = 'GET';
        $path = $this->rechargeUrl . '/customers?email='.$email;
        $response = $this->call($method, $path);
        return json_decode($response);
    }

    /**
     * get subscription by customer id
     */
    public function getSubscriptionByCustomerID($customerID)
    {

        $method = 'GET';
        $path = $this->rechargeUrl . '/subscriptions?customer_id='.$customerID;
        $response = $this->call($method, $path);
        return json_decode($response);
    }

    /**
     * get customer address by address id
     */
    public function getAddressByAddressID($addressId)
    {

        $method = 'GET';
        $path = env('RECHARGE_URL', true).'/addresses/'.$addressId;
        $response = $this->call($method, $path);
        return json_decode($response);
    }

    /**
     * get shipping rate by address
     */
    public function updateShippingRateByAddress($addressId, $data)
    {

        $method = 'PUT';
        $path = env('RECHARGE_URL', true).'/addresses/'.$addressId;
        $response = $this->call($method, $path, $data);
        return json_decode($response);
    }

    /**
     * get all subscription by customer id
     */
    public function getAllSubscriptionsByCustomerID($customerID)
    {

        $method = 'GET';
        $path = env('RECHARGE_URL', true).'/subscriptions?customer_id='.$customerID;
        $response = $this->call($method, $path);
        return json_decode($response);
    }

    /**
     * update subscription using upsell addon
     */
    public function updateUpsellSubscription($data = null)
    {

        $method = 'GET';
        $path = env('UPDATE_SUBSCRIPTION_URL', true).'updateSubscriptionLineItemProperties';
        $response = $this->callEndpoint($method, $path, $data);
        return json_decode($response);
    }

    /**
     * get all active subscriptions by customer id
     */
    public function getAllActiveSubscriptionsByCustomerID($customerID)
    {

        $method = 'GET';
        $path = env('RECHARGE_URL', true).'/subscriptions?customer_id='.$customerID.'&status=active';
        $response = $this->call($method, $path);
        return json_decode($response);
    }

    /**
     * get all recharge subscriptions
     */
    public function getAllSubscriptions($page = null, $limit = null)
    {

        $method = 'GET';
        $path = env('RECHARGE_URL', true).'/subscriptions?status=active&page='.$page.'&limit='.$limit;
        $response = $this->call($method, $path);
        return json_decode($response);
    }


    /**
     * get all recharge customers
     */
    public function getAllActiveCustomers($page = null, $limit = null)
    {

        $method = 'GET';
        $path = env('RECHARGE_URL', true).'/customers?status=active&page='.$page.'&limit='.$limit;
        $response = $this->call($method, $path);
        return json_decode($response);
    }
    
    /**
     * get charge details by subscriptionID
     */
    function getChargeDetailsBySubscriptionID($subscriptionID) {

        $method = 'GET';
        $path = env('RECHARGE_URL', true).'/charges?subscription_id='.$subscriptionID;
        $response = $this->call($method, $path);
        return json_decode($response);
    }

    /**
     * get discount details by discount code
     */
    function getDiscountDetailsByDiscountCode($discountCode) {

        $method = 'GET';
        $path = env('RECHARGE_URL', true).'/discounts?discount_code='.$discountCode;
        $response = $this->call($method, $path);
        return json_decode($response);
    }

    /**
     * apply discount to a charge by discount code
     */
    function applyDiscountToChargeByDiscountCode($chargeID, $data) {

        $method = 'POST';
        $path = env('RECHARGE_URL', true).'/charges/'.$chargeID.'/apply_discount';
        $response = $this->call($method, $path, $data);
        return json_decode($response);
    }
    
    /**
     * get charge details by addressID
     */
    function getOtpChargeDetails($addressID) {

        $method = 'GET';
        $path = env('RECHARGE_URL', true).'/charges?address_id='.$addressID;
        $response = $this->call($method, $path);
        return json_decode($response);
    }

    /**
     * create subscription in recharge
     */
    function createSubscription($data) {

        $method = 'POST';
        $path = env('RECHARGE_URL', true).'/subscriptions';
        $response = $this->call($method, $path, $data);
        return json_decode($response);
    }

    /**
     * get all discount details
     */
    function getDiscountDetails() {

        $method = 'GET';
        $path = env('RECHARGE_URL', true).'/discounts';
        $response = $this->call($method, $path);
        return json_decode($response);
    }

     /**
     * get charge by address_id details
     */
    function getChargeByAddressID($address_id) {

        $method = 'GET';
        $path = env('RECHARGE_URL', true).'/charges?address_id='.$address_id.'&status=QUEUED';
        $response = $this->call($method, $path);
        return json_decode($response);
    }

    /**
     * remove discount from an address
     */
    function removeDiscountFromAdress($address_id) {

        $method = 'POST';
        $path = env('RECHARGE_URL', true).'/addresses/'.$address_id.'/remove_discount';
        $response = $this->call($method, $path);
        return json_decode($response);
    }

    /**
     * get all discount details by id
     */
    function getDiscountDetailsByID($discount_id) {

        $method = 'GET';
        $path = env('RECHARGE_URL', true).'/discounts/'.$discount_id;
        $response = $this->call($method, $path);
        return json_decode($response);
    }

    /**
     * Update discount by id
     */
    function updateDiscount($discount_id, $data) {

        $method = 'PUT';
        $path = env('RECHARGE_URL', true).'/discounts/'.$discount_id;
        $response = $this->call($method, $path, $data);
        return json_decode($response);
    }

    /**
     * CURL call
     */
    public function call($method, $path, $data = [])
    {
        $curl = curl_init();
        curl_setopt($curl, CURLOPT_URL, $path);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_ENCODING, '');
        curl_setopt($curl, CURLOPT_MAXREDIRS, 10);
        curl_setopt($curl, CURLOPT_TIMEOUT, 30);
        curl_setopt($curl, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_1);
        curl_setopt($curl, CURLOPT_CUSTOMREQUEST, $method);

        if ($method !== 'GET') {
            curl_setopt($curl, CURLOPT_POSTFIELDS, $data);
        }

        curl_setopt($curl, CURLOPT_HTTPHEADER, array(
            "cache-control: no-cache",
            "x-recharge-access-token: " . env('RECHARGE_API_KEY', true),
            "content-type: application/json",
        ));

        $response = curl_exec($curl);
        return $response;
    }

    /**
     * CURL call for update the upsell subscription
     */
    public function callEndpoint($method, $path, $data = [])
    {

        $curl = curl_init();
        curl_setopt($curl, CURLOPT_URL, $path.'?callback=update&data='.urlencode($data));
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        //curl_setopt($curl, CURLOPT_MAXREDIRS, 10);
        //curl_setopt($curl, CURLOPT_TIMEOUT, 30);
        //curl_setopt($curl, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_1);
        curl_setopt($curl, CURLOPT_CUSTOMREQUEST, $method);

        curl_setopt($curl, CURLOPT_HTTPHEADER, array(
            "content-type: application/json",

        ));

        $response = curl_exec($curl);
        return $response;
    }
}
